@@ -149,6 +149,10 @@ class Backend:
             dirname = 'meson-out'
         return dirname
 
+    def get_target_source_dir(self, target):
+        dirname = os.path.join(self.build_to_src, self.get_target_dir(target))
+        return dirname
+
     def get_target_private_dir(self, target):
         dirname = os.path.join(self.get_target_dir(target), target.get_basename() + target.type_suffix())
         return dirname
@@ -220,7 +224,7 @@ class Backend:
                 raise MesonException('Unknown data type in object list.')
         return obj_list
 
-    def serialise_executable(self, exe, cmd_args, workdir, env={},
+    def serialize_executable(self, exe, cmd_args, workdir, env={},
                              capture=None):
         import hashlib
         # Can't just use exe.name here; it will likely be run more than once
@@ -265,7 +269,7 @@ class Backend:
             pickle.dump(es, f)
         return exe_data
 
-    def serialise_tests(self):
+    def serialize_tests(self):
         test_data = os.path.join(self.environment.get_scratch_dir(), 'meson_test_setup.dat')
         with open(test_data, 'wb') as datafile:
             self.write_test_file(datafile)
@@ -290,6 +294,15 @@ class Backend:
             raise MesonException(m.format(target.name))
         return l
 
+    def determine_rpath_dirs(self, target):
+        link_deps = target.get_all_link_deps()
+        result = []
+        for ld in link_deps:
+            prospective = self.get_target_dir(ld)
+            if prospective not in result:
+                result.append(prospective)
+        return result
+
     def object_filename_from_source(self, target, source, is_unity):
         if isinstance(source, mesonlib.File):
             source = source.fname
@@ -313,7 +326,9 @@ class Backend:
             # anyone both enables unity builds and has a file called foo-unity.cpp.
             osrc = self.get_unity_source_filename(extobj.target,
                                                   comp.get_default_suffix())
+            osrc = os.path.join(self.get_target_private_dir(extobj.target), osrc)
             objname = self.object_filename_from_source(extobj.target, osrc, True)
+            objname = objname.replace('/', '_').replace('\\', '_')
             objpath = os.path.join(proj_dir_to_build_root, targetdir, objname)
             return [objpath]
         for osrc in extobj.srclist:
@@ -328,7 +343,7 @@ class Backend:
         includeargs = compiler.get_include_args(pchpath, False)
         for lang in ['c', 'cpp']:
             p = target.get_pch(lang)
-            if len(p) == 0:
+            if not p:
                 continue
             if compiler.can_compile(p[-1]):
                 header = p[0]
@@ -399,9 +414,10 @@ class Backend:
         # Add compile args added using add_global_arguments()
         # These override per-project arguments
         commands += self.build.get_global_args(compiler)
-        # Compile args added from the env: CFLAGS/CXXFLAGS, etc. We want these
-        # to override all the defaults, but not the per-target compile args.
-        commands += self.environment.coredata.external_args[compiler.get_language()]
+        if not target.is_cross:
+            # Compile args added from the env: CFLAGS/CXXFLAGS, etc. We want these
+            # to override all the defaults, but not the per-target compile args.
+            commands += self.environment.coredata.external_args[compiler.get_language()]
         # Always set -fPIC for shared libraries
         if isinstance(target, build.SharedLibrary):
             commands += compiler.get_pic_args()
@@ -413,7 +429,21 @@ class Backend:
         # NOTE: We must preserve the order in which external deps are
         # specified, so we reverse the list before iterating over it.
         for dep in reversed(target.get_external_deps()):
-            commands += dep.get_compile_args()
+            if not dep.found():
+                continue
+
+            if compiler.language == 'vala':
+                if isinstance(dep, dependencies.PkgConfigDependency):
+                    if dep.name == 'glib-2.0' and dep.version_reqs is not None:
+                        for req in dep.version_reqs:
+                            if req.startswith(('>=', '==')):
+                                commands += ['--target-glib', req[2:]]
+                                break
+                    commands += ['--pkg', dep.name]
+                elif isinstance(dep, dependencies.ExternalLibrary):
+                    commands += dep.get_link_args('vala')
+            else:
+                commands += dep.get_compile_args()
             # Qt needs -fPIC for executables
             # XXX: We should move to -fPIC for all executables
             if isinstance(target, build.Executable):
@@ -435,15 +465,11 @@ class Backend:
         for d in deps:
             if not isinstance(d, (build.StaticLibrary, build.SharedLibrary)):
                 raise RuntimeError('Tried to link with a non-library target "%s".' % d.get_basename())
-            if isinstance(compiler, compilers.LLVMDCompiler) or isinstance(compiler, compilers.DmdDCompiler):
-                args += ['-L' + self.get_target_filename_for_linking(d)]
+            if isinstance(compiler, (compilers.LLVMDCompiler, compilers.DmdDCompiler)):
+                d_arg = '-L' + self.get_target_filename_for_linking(d)
             else:
-                args.append(self.get_target_filename_for_linking(d))
-            # If you have executable e that links to shared lib s1 that links to shared library s2
-            # you have to specify s2 as well as s1 when linking e even if e does not directly use
-            # s2. Gcc handles this case fine but Clang does not for some reason. Thus we need to
-            # explictly specify all libraries every time.
-            args += self.build_target_link_arguments(compiler, d.get_dependencies())
+                d_arg = self.get_target_filename_for_linking(d)
+            args.append(d_arg)
         return args
 
     def determine_windows_extra_paths(self, target):
@@ -512,9 +538,7 @@ class Backend:
             return
         ifilename = os.path.join(self.environment.get_build_dir(), 'depmf.json')
         ofilename = os.path.join(self.environment.get_prefix(), self.build.dep_manifest_name)
-        mfobj = {'type': 'dependency manifest',
-                 'version': '1.0'}
-        mfobj['projects'] = self.build.dep_manifest
+        mfobj = {'type': 'dependency manifest', 'version': '1.0', 'projects': self.build.dep_manifest}
         with open(ifilename, 'w') as f:
             f.write(json.dumps(mfobj))
         # Copy file from, to, and with mode unchanged
@@ -613,7 +637,6 @@ class Backend:
             return True
         return False
 
-
     def get_custom_target_sources(self, target):
         '''
         Custom target sources can be of various object types; strings, File,
