@@ -12,10 +12,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os, re, subprocess, platform
-from . import coredata
-from . import mesonlib
-from . import mlog
+import os
+import platform
+import re
+
 from .compilers import *
 from .mesonlib import EnvironmentException, Popen_safe
 import configparser
@@ -24,6 +24,16 @@ import shutil
 
 build_filename = 'meson.build'
 
+# Environment variables that each lang uses.
+cflags_mapping = {'c': 'CFLAGS',
+                  'cpp': 'CXXFLAGS',
+                  'objc': 'OBJCFLAGS',
+                  'objcpp': 'OBJCXXFLAGS',
+                  'fortran': 'FFLAGS',
+                  'd': 'DFLAGS',
+                  'vala': 'VALAFLAGS'}
+
+
 def find_coverage_tools():
     gcovr_exe = 'gcovr'
     lcov_exe = 'lcov'
@@ -102,7 +112,7 @@ def detect_windows_arch(compilers):
             platform = os.environ.get('Platform', 'x86').lower()
             if platform == 'x86':
                 return platform
-        if compiler.id == 'gcc' and compiler.has_define('__i386__'):
+        if compiler.id == 'gcc' and compiler.has_builtin_define('__i386__'):
             return 'x86'
     return os_arch
 
@@ -129,10 +139,10 @@ def detect_cpu_family(compilers):
         # to know is to check the compiler defines.
         for c in compilers.values():
             try:
-                if c.has_define('__i386__'):
+                if c.has_builtin_define('__i386__'):
                     return 'x86'
             except mesonlib.MesonException:
-                # Ignore compilers that do not support has_define.
+                # Ignore compilers that do not support has_builtin_define.
                 pass
         return 'x86_64'
     # Add fixes here as bugs are reported.
@@ -149,7 +159,7 @@ def detect_cpu(compilers):
         # Same check as above for cpu_family
         for c in compilers.values():
             try:
-                if c.has_define('__i386__'):
+                if c.has_builtin_define('__i386__'):
                     return 'i686' # All 64 bit cpus have at least this level of x86 support.
             except mesonlib.MesonException:
                 pass
@@ -158,7 +168,10 @@ def detect_cpu(compilers):
     return trial
 
 def detect_system():
-    return platform.system().lower()
+    system = platform.system().lower()
+    if system.startswith('cygwin'):
+        return 'cygwin'
+    return system
 
 
 def for_windows(is_cross, env):
@@ -173,6 +186,20 @@ def for_windows(is_cross, env):
         return env.cross_info.config['host_machine']['system'] == 'windows'
     return False
 
+
+def for_cygwin(is_cross, env):
+    """
+    Host machine is cygwin?
+
+    Note: 'host' is the machine on which compiled binaries will run
+    """
+    if not is_cross:
+        return mesonlib.is_cygwin()
+    elif env.cross_info.has_host():
+        return env.cross_info.config['host_machine']['system'] == 'cygwin'
+    return False
+
+
 def for_darwin(is_cross, env):
     """
     Host machine is Darwin (iOS/OS X)?
@@ -244,9 +271,11 @@ class Environment:
             self.default_cpp = ['c++']
         self.default_objc = ['cc']
         self.default_objcpp = ['c++']
-        self.default_fortran = ['gfortran', 'g95', 'f95', 'f90', 'f77']
-        self.default_static_linker = 'ar'
-        self.vs_static_linker = 'lib'
+        self.default_fortran = ['gfortran', 'g95', 'f95', 'f90', 'f77', 'ifort']
+        self.default_static_linker = ['ar']
+        self.vs_static_linker = ['lib']
+        self.gcc_static_linker = ['gcc-ar']
+        self.clang_static_linker = ['llvm-ar']
 
         # Various prefixes and suffixes for import libraries, shared libraries,
         # static libraries, and executables.
@@ -257,6 +286,11 @@ class Environment:
             self.exe_suffix = 'exe'
             self.object_suffix = 'obj'
             self.win_libdir_layout = True
+        elif (not cross and mesonlib.is_cygwin()) \
+                or (cross and self.cross_info.has_host() and self.cross_info.config['host_machine']['system'] == 'cygwin'):
+            self.exe_suffix = 'exe'
+            self.object_suffix = 'o'
+            self.win_libdir_layout = True
         else:
             self.exe_suffix = ''
             self.object_suffix = 'o'
@@ -368,7 +402,8 @@ class Environment:
             return GCC_OSX
         elif '__MINGW32__' in defines or '__MINGW64__' in defines:
             return GCC_MINGW
-        # We ignore Cygwin for now, and treat it as a standard GCC
+        elif '__CYGWIN__' in defines:
+            return GCC_CYGWIN
         return GCC_STANDARD
 
     def _get_compilers(self, lang, evar, want_cross):
@@ -377,16 +412,30 @@ class Environment:
         C, C++, ObjC, ObjC++, Fortran so consolidate it here.
         '''
         if self.is_cross_build() and want_cross:
-            compilers = [mesonlib.stringlistify(self.cross_info.config['binaries'][lang])]
-            ccache = []
+            compilers = mesonlib.stringlistify(self.cross_info.config['binaries'][lang])
+            # Ensure ccache exists and remove it if it doesn't
+            if compilers[0] == 'ccache':
+                compilers = compilers[1:]
+                ccache = self.detect_ccache()
+            else:
+                ccache = []
+            # Return value has to be a list of compiler 'choices'
+            compilers = [compilers]
             is_cross = True
             if self.cross_info.need_exe_wrapper():
                 exe_wrap = self.cross_info.config['binaries'].get('exe_wrapper', None)
             else:
                 exe_wrap = []
         elif evar in os.environ:
-            compilers = [shlex.split(os.environ[evar])]
-            ccache = []
+            compilers = shlex.split(os.environ[evar])
+            # Ensure ccache exists and remove it if it doesn't
+            if compilers[0] == 'ccache':
+                compilers = compilers[1:]
+                ccache = self.detect_ccache()
+            else:
+                ccache = []
+            # Return value has to be a list of compiler 'choices'
+            compilers = [compilers]
             is_cross = False
             exe_wrap = None
         else:
@@ -396,8 +445,8 @@ class Environment:
             exe_wrap = None
         return compilers, ccache, is_cross, exe_wrap
 
-    def _handle_compiler_exceptions(self, exceptions, compilers):
-        errmsg = 'Unknown compiler(s): ' + str(compilers)
+    def _handle_exceptions(self, exceptions, binaries, bintype='compiler'):
+        errmsg = 'Unknown {}(s): {}'.format(bintype, binaries)
         if exceptions:
             errmsg += '\nThe follow exceptions were encountered:'
             for (c, e) in exceptions.items():
@@ -410,11 +459,11 @@ class Environment:
         for compiler in compilers:
             if isinstance(compiler, str):
                 compiler = [compiler]
+            if 'cl' in compiler or 'cl.exe' in compiler:
+                arg = '/?'
+            else:
+                arg = '--version'
             try:
-                if 'cl' in compiler or 'cl.exe' in compiler:
-                    arg = '/?'
-                else:
-                    arg = '--version'
                 p, out, err = Popen_safe(compiler + [arg])
             except OSError as e:
                 popen_exceptions[' '.join(compiler + [arg])] = e
@@ -432,6 +481,8 @@ class Environment:
             if 'clang' in out:
                 if 'Apple' in out or for_darwin(want_cross, self):
                     cltype = CLANG_OSX
+                elif 'windows' in out or for_windows(want_cross, self):
+                    cltype = CLANG_WIN
                 else:
                     cltype = CLANG_STANDARD
                 cls = ClangCCompiler if lang == 'c' else ClangCPPCompiler
@@ -447,7 +498,7 @@ class Environment:
                 inteltype = ICC_STANDARD
                 cls = IntelCCompiler if lang == 'c' else IntelCPPCompiler
                 return cls(ccache + compiler, version, inteltype, is_cross, exe_wrap)
-        self._handle_compiler_exceptions(popen_exceptions, compilers)
+        self._handle_exceptions(popen_exceptions, compilers)
 
     def detect_c_compiler(self, want_cross):
         return self._detect_c_or_cpp_compiler('c', 'CC', want_cross)
@@ -500,7 +551,7 @@ class Environment:
 
                 if 'NAG Fortran' in err:
                     return NAGFortranCompiler(compiler, version, is_cross, exe_wrap)
-        self._handle_compiler_exceptions(popen_exceptions, compilers)
+        self._handle_exceptions(popen_exceptions, compilers)
 
     def get_scratch_dir(self):
         return self.scratch_dir
@@ -533,7 +584,7 @@ class Environment:
                 return ClangObjCCompiler(ccache + compiler, version, CLANG_OSX, is_cross, exe_wrap)
             if out.startswith('clang'):
                 return ClangObjCCompiler(ccache + compiler, version, CLANG_STANDARD, is_cross, exe_wrap)
-        self._handle_compiler_exceptions(popen_exceptions, compilers)
+        self._handle_exceptions(popen_exceptions, compilers)
 
     def detect_objcpp_compiler(self, want_cross):
         popen_exceptions = {}
@@ -559,7 +610,7 @@ class Environment:
                 return ClangObjCPPCompiler(ccache + compiler, version, CLANG_OSX, is_cross, exe_wrap)
             if out.startswith('clang'):
                 return ClangObjCPPCompiler(ccache + compiler, version, CLANG_STANDARD, is_cross, exe_wrap)
-        self._handle_compiler_exceptions(popen_exceptions, compilers)
+        self._handle_exceptions(popen_exceptions, compilers)
 
     def detect_java_compiler(self):
         exelist = ['javac']
@@ -656,29 +707,40 @@ class Environment:
             linker = self.cross_info.config['binaries']['ar']
             if isinstance(linker, str):
                 linker = [linker]
+            linkers = [linker]
         else:
             evar = 'AR'
             if evar in os.environ:
-                linker = shlex.split(os.environ[evar])
+                linkers = [shlex.split(os.environ[evar])]
             elif isinstance(compiler, VisualStudioCCompiler):
-                linker = [self.vs_static_linker]
+                linkers = [self.vs_static_linker]
+            elif isinstance(compiler, GnuCompiler):
+                # Use gcc-ar if available; needed for LTO
+                linkers = [self.gcc_static_linker, self.default_static_linker]
+            elif isinstance(compiler, ClangCompiler):
+                # Use llvm-ar if available; needed for LTO
+                linkers = [self.clang_static_linker, self.default_static_linker]
             else:
-                linker = [self.default_static_linker]
-        if 'lib' in linker or 'lib.exe' in linker:
-            arg = '/?'
-        else:
-            arg = '--version'
-        try:
-            p, out, err = Popen_safe(linker + [arg])
-        except OSError:
-            raise EnvironmentException('Could not execute static linker "%s".' % ' '.join(linker))
-        if '/OUT:' in out or '/OUT:' in err:
-            return VisualStudioLinker(linker)
-        if p.returncode == 0:
-            return ArLinker(linker)
-        if p.returncode == 1 and err.startswith('usage'): # OSX
-            return ArLinker(linker)
-        raise EnvironmentException('Unknown static linker "%s"' % ' '.join(linker))
+                linkers = [self.default_static_linker]
+        popen_exceptions = {}
+        for linker in linkers:
+            if 'lib' in linker or 'lib.exe' in linker:
+                arg = '/?'
+            else:
+                arg = '--version'
+            try:
+                p, out, err = Popen_safe(linker + [arg])
+            except OSError as e:
+                popen_exceptions[' '.join(linker + [arg])] = e
+                continue
+            if '/OUT:' in out or '/OUT:' in err:
+                return VisualStudioLinker(linker)
+            if p.returncode == 0:
+                return ArLinker(linker)
+            if p.returncode == 1 and err.startswith('usage'): # OSX
+                return ArLinker(linker)
+        self._handle_exceptions(popen_exceptions, linkers, 'linker')
+        raise EnvironmentException('Unknown static linker "%s"' % ' '.join(linkers))
 
     def detect_ccache(self):
         try:
@@ -704,6 +766,10 @@ class Environment:
         "Install dir for the import library (library used for linking)"
         return self.get_libdir()
 
+    def get_shared_module_dir(self):
+        "Install dir for shared modules that are loaded at runtime"
+        return self.get_libdir()
+
     def get_shared_lib_dir(self):
         "Install dir for the shared library"
         if self.win_libdir_layout:
@@ -755,24 +821,17 @@ def get_args_from_envvars(compiler):
     if hasattr(compiler, 'get_linker_exelist'):
         compiler_is_linker = (compiler.get_exelist() == compiler.get_linker_exelist())
 
-    if lang not in ('c', 'cpp', 'objc', 'objcpp', 'fortran', 'd'):
-        return [], []
+    if lang not in cflags_mapping:
+        return [], [], []
 
-    # Compile flags
-    cflags_mapping = {'c': 'CFLAGS',
-                      'cpp': 'CXXFLAGS',
-                      'objc': 'OBJCFLAGS',
-                      'objcpp': 'OBJCXXFLAGS',
-                      'fortran': 'FFLAGS',
-                      'd': 'DFLAGS'}
     compile_flags = os.environ.get(cflags_mapping[lang], '')
     log_var(cflags_mapping[lang], compile_flags)
-    compile_flags = compile_flags.split()
+    compile_flags = shlex.split(compile_flags)
 
     # Link flags (same for all languages)
     link_flags = os.environ.get('LDFLAGS', '')
     log_var('LDFLAGS', link_flags)
-    link_flags = link_flags.split()
+    link_flags = shlex.split(link_flags)
     if compiler_is_linker:
         # When the compiler is used as a wrapper around the linker (such as
         # with GCC and Clang), the compile flags can be needed while linking
@@ -780,14 +839,15 @@ def get_args_from_envvars(compiler):
         # this when the linker is stand-alone such as with MSVC C/C++, etc.
         link_flags = compile_flags + link_flags
 
-    # Pre-processof rlags (not for fortran)
+    # Pre-processor flags (not for fortran or D)
     preproc_flags = ''
     if lang in ('c', 'cpp', 'objc', 'objcpp'):
         preproc_flags = os.environ.get('CPPFLAGS', '')
     log_var('CPPFLAGS', preproc_flags)
-    compile_flags += preproc_flags.split()
+    preproc_flags = shlex.split(preproc_flags)
+    compile_flags += preproc_flags
 
-    return compile_flags, link_flags
+    return preproc_flags, compile_flags, link_flags
 
 class CrossBuildInfo:
     def __init__(self, filename):
@@ -814,7 +874,7 @@ class CrossBuildInfo:
                 if ' ' in entry or '\t' in entry or "'" in entry or '"' in entry:
                     raise EnvironmentException('Malformed variable name %s in cross file..' % entry)
                 try:
-                    res = eval(value, {'true': True, 'false': False})
+                    res = eval(value, {'__builtins__': None}, {'true': True, 'false': False})
                 except Exception:
                     raise EnvironmentException('Malformed value in cross file variable %s.' % entry)
                 if self.ok_type(res):
@@ -848,6 +908,9 @@ class CrossBuildInfo:
         return 'host_machine' in self.config
 
     def need_exe_wrapper(self):
+        value = self.config['properties'].get('needs_exe_wrapper', None)
+        if value is not None:
+            return value
         # Can almost always run 32-bit binaries on 64-bit natively if the host
         # and build systems are the same. We don't pass any compilers to
         # detect_cpu_family() here because we always want to know the OS
