@@ -15,6 +15,7 @@
 """A library of random helper functionality."""
 
 import stat
+import time
 import platform, subprocess, operator, os, shutil, re
 import collections
 
@@ -203,7 +204,7 @@ def classify_unity_sources(compilers, sources):
 
 def flatten(item):
     if not isinstance(item, list):
-        return item
+        return [item]
     result = []
     for i in item:
         if isinstance(i, list):
@@ -305,7 +306,7 @@ def version_compare(vstr1, vstr2, strict=False):
     return cmpop(varr1, varr2)
 
 def version_compare_many(vstr1, conditions):
-    if not isinstance(conditions, (list, tuple)):
+    if not isinstance(conditions, (list, tuple, frozenset)):
         conditions = [conditions]
     found = []
     not_found = []
@@ -371,7 +372,7 @@ def do_replacement(regex, line, confdata):
     match = re.search(regex, line)
     while match:
         varname = match.group(1)
-        if varname in confdata.keys():
+        if varname in confdata:
             (var, desc) = confdata.get(varname)
             if isinstance(var, str):
                 pass
@@ -687,17 +688,57 @@ def get_filenames_templates_dict(inputs, outputs):
             values['@OUTDIR@'] = '.'
     return values
 
-class OrderedSet(collections.OrderedDict):
-    '''
-    A 'set' equivalent that preserves the order in which items are added.
 
-    This is a hack implementation that wraps OrderedDict. It may not be the
-    most efficient solution and might need fixing to override more methods.
-    '''
+def windows_proof_rmtree(f):
+    # On Windows if anyone is holding a file open you can't
+    # delete it. As an example an anti virus scanner might
+    # be scanning files you are trying to delete. The only
+    # way to fix this is to try again and again.
+    delays = [0.1, 0.1, 0.2, 0.2, 0.2, 0.5, 0.5, 1, 1, 1, 1, 2]
+    for d in delays:
+        try:
+            shutil.rmtree(f)
+            return
+        except (OSError, PermissionError):
+            time.sleep(d)
+    # Try one last time and throw if it fails.
+    shutil.rmtree(f)
+
+class OrderedSet(collections.MutableSet):
+    """A set that preserves the order in which items are added, by first
+    insertion.
+    """
     def __init__(self, iterable=None):
+        self.__container = collections.OrderedDict()
         if iterable:
             self.update(iterable)
 
+    def __contains__(self, value):
+        return value in self.__container
+
+    def __iter__(self):
+        return iter(self.__container.keys())
+
+    def __len__(self):
+        return len(self.__container)
+
+    def __repr__(self):
+        # Don't print 'OrderedSet("")' for an empty set.
+        if self.__container:
+            return 'OrderedSet("{}")'.format(
+                '", "'.join(repr(e) for e in self.__container.keys()))
+        return 'OrderedSet()'
+
+    def add(self, value):
+        self.__container[value] = None
+
+    def discard(self, value):
+        if value in self.__container:
+            del self.__container[value]
+
     def update(self, iterable):
         for item in iterable:
-            self[item] = True
+            self.__container[item] = None
+
+    def difference(self, set_):
+        return type(self)(e for e in self if e not in set_)
