@@ -20,6 +20,7 @@ from . import build
 import platform
 from . import mlog, coredata
 from .mesonlib import MesonException
+from .wrap import WrapMode
 
 
 parser = argparse.ArgumentParser()
@@ -53,7 +54,7 @@ add_builtin_argument('sharedstatedir')
 add_builtin_argument('backend')
 add_builtin_argument('buildtype')
 add_builtin_argument('strip', action='store_true')
-add_builtin_argument('unity', action='store_true')
+add_builtin_argument('unity')
 add_builtin_argument('werror', action='store_true')
 add_builtin_argument('layout')
 add_builtin_argument('default-library')
@@ -63,10 +64,14 @@ add_builtin_argument('errorlogs', action='store_false')
 
 parser.add_argument('--cross-file', default=None,
                     help='File describing cross compilation environment.')
-parser.add_argument('-D', action='append', dest='projectoptions', default=[],
-                    help='Set project options.')
+parser.add_argument('-D', action='append', dest='projectoptions', default=[], metavar="option",
+                    help='Set the value of an option, can be used several times to set multiple options.')
 parser.add_argument('-v', '--version', action='version',
                     version=coredata.version)
+# See the mesonlib.WrapMode enum for documentation
+parser.add_argument('--wrap-mode', default=WrapMode.default,
+                    type=lambda t: getattr(WrapMode, t), choices=WrapMode,
+                    help='Special wrap mode to use')
 parser.add_argument('directories', nargs='*')
 
 class MesonApp:
@@ -82,8 +87,8 @@ class MesonApp:
         return os.path.exists(fname)
 
     def validate_core_dirs(self, dir1, dir2):
-        ndir1 = os.path.abspath(dir1)
-        ndir2 = os.path.abspath(dir2)
+        ndir1 = os.path.abspath(os.path.realpath(dir1))
+        ndir2 = os.path.abspath(os.path.realpath(dir2))
         if not os.path.exists(ndir1):
             os.makedirs(ndir1)
         if not os.path.exists(ndir2):
@@ -145,12 +150,19 @@ If you want to change option values, use the mesonconf tool instead.'''
         if self.options.backend == 'ninja':
             from .backend import ninjabackend
             g = ninjabackend.NinjaBackend(b)
+        elif self.options.backend == 'vs':
+            from .backend import vs2010backend
+            g = vs2010backend.autodetect_vs_version(b)
+            mlog.log('Auto detected Visual Studio backend:', mlog.bold(g.name))
         elif self.options.backend == 'vs2010':
             from .backend import vs2010backend
             g = vs2010backend.Vs2010Backend(b)
         elif self.options.backend == 'vs2015':
             from .backend import vs2015backend
             g = vs2015backend.Vs2015Backend(b)
+        elif self.options.backend == 'vs2017':
+            from .backend import vs2017backend
+            g = vs2017backend.Vs2017Backend(b)
         elif self.options.backend == 'xcode':
             from .backend import xcodebackend
             g = xcodebackend.XCodeBackend(b)
@@ -210,6 +222,9 @@ def run_script_command(args):
     elif cmdname == 'gtkdoc':
         import mesonbuild.scripts.gtkdochelper as abc
         cmdfunc = abc.run
+    elif cmdname == 'msgfmthelper':
+        import mesonbuild.scripts.msgfmthelper as abc
+        cmdfunc = abc.run
     elif cmdname == 'regencheck':
         import mesonbuild.scripts.regen_checker as abc
         cmdfunc = abc.run
@@ -231,6 +246,12 @@ def run_script_command(args):
     elif cmdname == 'uninstall':
         import mesonbuild.scripts.uninstall as abc
         cmdfunc = abc.run
+    elif cmdname == 'dist':
+        import mesonbuild.scripts.dist as abc
+        cmdfunc = abc.run
+    elif cmdname == 'coverage':
+        import mesonbuild.scripts.coverage as abc
+        cmdfunc = abc.run
     else:
         raise MesonException('Unknown internal command {}.'.format(cmdname))
     return cmdfunc(cmdargs)
@@ -257,10 +278,10 @@ def run(mainfile, args):
     args = mesonlib.expand_arguments(args)
     options = parser.parse_args(args)
     args = options.directories
-    if len(args) == 0 or len(args) > 2:
+    if not args or len(args) > 2:
         # if there's a meson.build in the dir above, and not in the current
         # directory, assume we're in the build directory
-        if len(args) == 0 and not os.path.exists('meson.build') and os.path.exists('../meson.build'):
+        if not args and not os.path.exists('meson.build') and os.path.exists('../meson.build'):
             dir1 = '..'
             dir2 = '.'
         else:
@@ -291,7 +312,11 @@ def run(mainfile, args):
             else:
                 mlog.log(mlog.red('\nMeson encountered an error:'))
             mlog.log(e)
+            if os.environ.get('MESON_FORCE_BACKTRACE'):
+                raise
         else:
+            if os.environ.get('MESON_FORCE_BACKTRACE'):
+                raise
             traceback.print_exc()
         return 1
     return 0
