@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import sys, pickle, os, shutil, subprocess, gzip, platform
+import sys, pickle, os, shutil, subprocess, gzip, platform, errno
 from glob import glob
 from . import depfixer
 from . import destdir_join
@@ -34,6 +34,15 @@ def set_mode(path, mode):
         except PermissionError as e:
             msg = '{!r}: Unable to set owner {!r} and group {!r}: {}, ignoring...'
             print(msg.format(path, mode.owner, mode.group, e.strerror))
+        except LookupError:
+            msg = '{!r}: Non-existent owner {!r} or group {!r}: ignoring...'
+            print(msg.format(path, mode.owner, mode.group))
+        except OSError as e:
+            if e.errno == errno.EINVAL:
+                msg = '{!r}: Non-existent numeric owner {!r} or group {!r}: ignoring...'
+                print(msg.format(path, mode.owner, mode.group))
+            else:
+                raise
     # Must set permissions *after* setting owner/group otherwise the
     # setuid/setgid bits will get wiped by chmod
     # NOTE: On Windows you can set read/write perms; the rest are ignored
@@ -124,7 +133,7 @@ def install_subdirs(data):
         if src_dir.endswith('/') or src_dir.endswith('\\'):
             src_dir = src_dir[:-1]
         src_prefix = os.path.join(src_dir, inst_dir)
-        print('Installing subdir %s to %s.' % (src_prefix, dst_dir))
+        print('Installing subdir %s to %s' % (src_prefix, dst_dir))
         dst_dir = get_destdir_path(data, dst_dir)
         if not os.path.exists(dst_dir):
             os.makedirs(dst_dir)
@@ -139,7 +148,7 @@ def install_data(d):
         mode = i[2]
         outdir = os.path.split(outfilename)[0]
         os.makedirs(outdir, exist_ok=True)
-        print('Installing %s to %s.' % (fullfilename, outdir))
+        print('Installing %s to %s' % (fullfilename, outdir))
         do_copyfile(fullfilename, outfilename)
         set_mode(outfilename, mode)
 
@@ -149,7 +158,7 @@ def install_man(d):
         outfilename = get_destdir_path(d, m[1])
         outdir = os.path.split(outfilename)[0]
         os.makedirs(outdir, exist_ok=True)
-        print('Installing %s to %s.' % (full_source_filename, outdir))
+        print('Installing %s to %s' % (full_source_filename, outdir))
         if outfilename.endswith('.gz') and not full_source_filename.endswith('.gz'):
             with open(outfilename, 'wb') as of:
                 with open(full_source_filename, 'rb') as sf:
@@ -174,7 +183,7 @@ def run_install_script(d):
            'MESON_BUILD_ROOT': d.build_dir,
            'MESON_INSTALL_PREFIX': d.prefix,
            'MESON_INSTALL_DESTDIR_PREFIX': d.fullprefix,
-           }
+           'MESONINTROSPECT': d.mesonintrospect}
     child_env = os.environ.copy()
     child_env.update(env)
 
@@ -193,7 +202,7 @@ def run_install_script(d):
 
 def is_elf_platform():
     platname = platform.system().lower()
-    if platname == 'darwin' or platname == 'windows':
+    if platname == 'darwin' or platname == 'windows' or platname == 'cygwin':
         return False
     return True
 
@@ -236,17 +245,22 @@ def install_targets(d):
             raise RuntimeError('File {!r} could not be found'.format(fname))
         elif os.path.isfile(fname):
             do_copyfile(fname, outname)
-            if should_strip:
+            if should_strip and d.strip_bin is not None:
                 if fname.endswith('.jar'):
                     print('Not stripping jar target:', os.path.split(fname)[1])
                     continue
                 print('Stripping target {!r}'.format(fname))
-                ps, stdo, stde = Popen_safe(['strip', outname])
+                ps, stdo, stde = Popen_safe(d.strip_bin + [outname])
                 if ps.returncode != 0:
                     print('Could not strip file.\n')
                     print('Stdout:\n%s\n' % stdo)
                     print('Stderr:\n%s\n' % stde)
                     sys.exit(1)
+            pdb_filename = os.path.splitext(fname)[0] + '.pdb'
+            if not should_strip and os.path.exists(pdb_filename):
+                pdb_outname = os.path.splitext(outname)[0] + '.pdb'
+                print('Installing pdb file %s to %s' % (pdb_filename, pdb_outname))
+                do_copyfile(pdb_filename, pdb_outname)
         elif os.path.isdir(fname):
             fname = os.path.join(d.build_dir, fname.rstrip('/'))
             do_copydir(fname, os.path.dirname(fname), outdir)
